==> freetalk_0.5.orig.tar.gz: freetalk-0.5.orig/src/freetalk.c <==
/*
  (C) 2005 Vikas Gorur <vikasgp@gmail.com>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.
    
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
    
  You should have received a copy of the GNU General Public
  License along with this program; if not, write to the Free
  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301 USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <argp.h>

#include <glib.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <loudmouth/loudmouth.h>
#include <guile/gh.h>
#include <getopt.h>

#include "callbacks.h"
#include "util.h"
#include "freetalk.h"
#include "roster.h"
#include "interpreter.h"
#include "commands.h"
#include "extensions.h"

ft_state state;

static void
state_init (void)
{
  do_set_prompt ("~\\/~ ");
  state.roster = NULL;
  state.script = NULL;
  state.config_dir = calloc (1, strlen (getenv ("HOME")) + 1 + strlen (".freetalk") + 1);
  sprintf (state.config_dir, "%s/.freetalk", getenv ("HOME"));
  state.need_ssl = 1; /* turn on ssl by default */
}

scm_t_catch_handler catcher ()
{
  PRINTF("No such command or buddy. See /help");
  return NULL;
}

static void
process_line (char *line)
{
  char *eval_str;

  if (!line)
    return;

  if (*line) {
    add_history (line);
  } else {
    do_set_current_buddy (NULL);
  }

  eval_str = strdup (line);

  state.async_printf = 0;
  if (interpreter (line) != 0) {
    gh_eval_str_with_catch (eval_str, (scm_t_catch_handler) catcher);
    scm_force_output (scm_current_output_port ());
  }
  g_free (eval_str);
  state.async_printf = 1;
}

gboolean
stdin_input_cb (GIOChannel *chan, GIOCondition *cond, gpointer conn)
{
  rl_callback_read_char ();
  return TRUE;
}

static int
insert_current_buddy (void)
{
  if (state.current_buddy) {
    rl_insert_text (state.current_buddy);
    rl_insert_text (" ");
    rl_redisplay ();
  }
  return 0;
}

void
interface_init (void)
{
  GIOChannel *chan;
  
  interpreter_init ();
  
  rl_callback_handler_install (state.prompt, process_line);
  rl_pre_input_hook = insert_current_buddy;
  rl_attempted_completion_function = (CPPFunction *) ft_auto_complete;
  rl_completion_entry_function = complete_none;
  
  state.async_printf = 1;

  chan = g_io_channel_unix_new (0);
  g_io_add_watch (chan, G_IO_IN, (GIOFunc) stdin_input_cb, NULL);
}

const char *argp_program_version = PACKAGE_NAME " " PACKAGE_VERSION;
const char *argp_program_bug_address = PACKAGE_BUGREPORT;

static error_t
parse_opts (int key, char *arg, struct argp_state *_state)
{
  switch (key) {
  case 'j':
    do_set_jid (arg);
    break;
  case 's':
    state.script = arg;
    break;
  default:
    /* hack to allow args to script */
    if (!state.script)
      return ARGP_ERR_UNKNOWN;
  }

  return 0;
}

static void
mode_init (void)
{
  int i = 1;

  while (i < state.argc) {
    if (!strcmp (state.argv[i], "-s")) {
      if (state.argv[i+1])
	state.script = state.argv[i+1];
    }
    i++;
  }
}

static void
args_init (void)
{
  struct {
    char *f[2];
  } f;
  static char doc[] = "Freetalk is a console based jabber client/bot with a readline interface and guile extensions";
  static char argp_doc[] = " ";
  static struct argp_option options [] = {
    {"jid", 'j', "JABBERID", 0, "user@domain Jabber ID" },
    {"script", 's', "SCRIPTFILE", 0, "Freetalk script" },
    { 0, }
  };
  static struct argp argp = { options, parse_opts, argp_doc, doc };

  argp_parse (&argp, state.argc, state.argv, 0, 0, &f);
}

static void
inner_main (int argc, char **argv)
{
  check_first_run ();

  state_init ();
  mode_init ();

  extensions_init ();
  //loudscream_init ();
  
  if (!state.script) {
    load_default_config (); /* ~/.freetalk/freetalk.scm */
    args_init ();
    ft_load ("init.scm");
    ft_load ("login.scm");

    interface_init ();

    do_main_loop ();
  } else {
    ft_load (state.script);
  }
}

/*
 * Debugging hint: use LM_DEBUG=NET ./freetalk to see all the traffic on the wire
 */

int
main (int argc, char **argv)
{
  state.argc = argc;
  state.argv = argv;

  gh_enter (argc, argv, inner_main);
  return 0;
}

==> freetalk-0.42.tar.gz: freetalk-0.42/src/freetalk.c <==
/*
  (C) 2005 Vikas Gorur <vikasgp@gmail.com>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.
    
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.
    
  You should have received a copy of the GNU General Public
  License along with this program; if not, write to the Free
  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA 02110-1301 USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <argp.h>

#include <glib.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <loudmouth/loudmouth.h>
#include <guile/gh.h>
#include <getopt.h>

#include "callbacks.h"
#include "util.h"
#include "freetalk.h"
#include "roster.h"
#include "interpreter.h"
#include "commands.h"
#include "extensions.h"

ft_state state;

static void
state_init (void)
{
  do_set_prompt ("~\\/~ ");
  state.roster = NULL;
  state.script = NULL;
  state.config_dir = calloc (1, strlen (getenv ("HOME")) + 1 + strlen (".freetalk") + 1);
  sprintf (state.config_dir, "%s/.freetalk", getenv ("HOME"));
  state.need_ssl = 1; /* turn on ssl by default */
}

static void
process_line (char *line)
{
  char *eval_str;

  if (!line)
    return;

  if (*line) {
    add_history (line);
  } else {
    do_set_current_buddy (NULL);
  }

  eval_str = strdup (line);

  state.async_printf = 0;
  if (interpreter (line) != 0) {
    gh_eval_str_with_stack_saving_handler (eval_str);
    scm_force_output (scm_current_output_port ());
  }
  g_free (eval_str);
  state.async_printf = 1;
}

gboolean
stdin_input_cb (GIOChannel *chan, GIOCondition *cond, gpointer conn)
{
  rl_callback_read_char ();
  return TRUE;
}

static int
insert_current_buddy (void)
{
  if (state.current_buddy) {
    rl_insert_text (state.current_buddy);
    rl_insert_text (" ");
    rl_redisplay ();
  }
  return 0;
}

void
interface_init (void)
{
  GIOChannel *chan;
  
  interpreter_init ();
  
  rl_callback_handler_install (state.prompt, process_line);
  rl_pre_input_hook = insert_current_buddy;
  rl_attempted_completion_function = (CPPFunction *) ft_auto_complete;
  rl_completion_entry_function = complete_none;
  
  state.async_printf = 1;

  chan = g_io_channel_unix_new (0);
  g_io_add_watch (chan, G_IO_IN, (GIOFunc) stdin_input_cb, NULL);
}

const char *argp_program_version = PACKAGE_NAME " " PACKAGE_VERSION;
const char *argp_program_bug_address = PACKAGE_BUGREPORT;

static error_t
parse_opts (int key, char *arg, struct argp_state *_state)
{
  switch (key) {
  case 'j':
    do_set_jid (arg);
    break;
  case 's':
    state.script = arg;
    break;
  default:
    /* hack to allow args to script */
    if (!state.script)
      return ARGP_ERR_UNKNOWN;
  }

  return 0;
}

static void
mode_init (void)
{
  int i = 1;

  while (i < state.argc) {
    if (!strcmp (state.argv[i], "-s")) {
      if (state.argv[i+1])
	state.script = state.argv[i+1];
    }
    i++;
  }
}

static void
args_init (void)
{
  struct {
    char *f[2];
  } f;
  static char doc[] = "Freetalk is a console based jabber client/bot with a readline interface and guile extensions";
  static char argp_doc[] = " ";
  static struct argp_option options [] = {
    {"jid", 'j', "JABBERID", 0, "user@domain Jabber ID" },
    {"script", 's', "SCRIPTFILE", 0, "Freetalk script" },
    { 0, }
  };
  static struct argp argp = { options, parse_opts, argp_doc, doc };

  argp_parse (&argp, state.argc, state.argv, 0, 0, &f);
}

static void
inner_main (int argc, char **argv)
{
  check_first_run ();

  state_init ();
  mode_init ();

  if (!state.script) {
    load_default_config (); /* ~/.freetalk/freetalk.scm */
    args_init ();
    ft_load ("init.scm");
    ft_load ("login.scm");

    interface_init ();

    do_main_loop ();
  } else {
    ft_load (state.script);
  }

  extensions_init ();
}

/*
 * Debugging hint: use LM_DEBUG=NET ./freetalk to see all the traffic on the wire
 */

int
main (int argc, char **argv)
{
  state.argc = argc;
  state.argv = argv;

  gh_enter (argc, argv, inner_main);
  return 0;
}
